// *** WARNING *** All data used by the 'rewritable' (aka 'user') code must be separated
// into a separate segment using the USERDATA macro. This prevents new variables in the
// user code from relocating variables used by the code which is NOT rewritten.
// *** ALSO NOTE *** USERDATA variables are never initialised by the C startup code, so
// a declaration of the form "USERDATA int fred = 5;" does not have the expected effect -
// the value of fred at startup is undefined, and the "=5" part is silently ignored. USERDATA
// variable must be explicitly initialised before use.
 
USERDATA unsigned char I2C_adr;
USERDATA unsigned char byte_adr;
USERDATA unsigned char SI570_data[6];

#define _2(x) (1UL << x)
#define f_min (4850*_2(19))	//min VCO frequency 4850 MHz*2^19
#define f_max (5670*_2(19))	//max VCO frequency 5670 MHz*2
#define f_av  (5260*_2(19))	//average VCO frequency
#define f freq.qw			//desired frequency
#define RFREQ rfreq._.w1_4	//upper 30 bits
#define _RFREQ rfreq._.w0	//lower 8 bits
#undef  fcryst
#define fcryst fcr.qw

USERDATA freq_t freq ;	//frequency [MHz]*2^21
USERDATA freq_t fcr ;   //crystal frequency [MHz]*2^24fs
						
USERDATA unsigned long frequency;

struct char_long
{
unsigned char w0;
unsigned long w1_4;
};

union 
{
struct char_long _;
unsigned char w[5];
} rfreq USERDATA;			//make 38 RFREQ bits accessible in different ways

USERDATA unsigned char	N;					//slow divider 
USERDATA unsigned char	HS_DIV;				//fast divider 
USERDATA unsigned long   debug;
//DG8SAQ
unsigned char SI570CalcRegs(void)  REWRITABLE;
unsigned char SI570CalcRegs(void){	//calculates SI570 registers from frequency*2^21, 34000clocks = 2ms required
	unsigned long x;
	unsigned long vco;					//VCO frequency [MHz]*2^19
	unsigned long N0;
	unsigned char valid=0;
	uint16_t _N;
	unsigned char _HS_DIV;
	unsigned char i,d;
	long fcryst_tot=fcryst+dfcryst;

	vco=f_max;
	N0=f_av/(f/8); 					//(f/2/4): last binary digit is 0.5 digit for rounding, note different scalings of f and f_av
	_HS_DIV=11;
	while (_HS_DIV>3)
		{
		_N=(uint16_t)N0/_HS_DIV;	//last digit of N is 0.5 digit for rounding
		_N=(_N+1)>>1;				//rounded to the nearest integer 
		if (_N>2) _N&=0xFE;			//N=1 or even
		if (_N>128) _N=128;			//N<=2^7
		if (_N!=1)					//calculate VCO frequency for these divider values, avoid overflows
			{						//better numerical accuracy, when cases are handled separately	
			x=f/2;					//f needs to be divided by 4 due to different scaling
			x=x*_HS_DIV;	
			x=x*(_N/2);				//_N even here
			}
		else
			{
			x=f/4;					
			x=x*_HS_DIV;
			}
		if ((x>=f_min)&(x<=vco))	//find best divider values with vco smallest but still alowed
		    {
			valid=1;
			vco=x;
			N=_N;
			HS_DIV=_HS_DIV;
			}

		_HS_DIV--;
		if ((_HS_DIV==10)|(_HS_DIV==8)) _HS_DIV--;	//skip unavailable divider ratios
		}
	if (valid>0)
		{
		////////////////debug	
		debug=vco/_2(19);
		////////////////debug ende
		N=N-1;						//convert divider ratio to SI570 register value
		HS_DIV=HS_DIV-4;			//convert divider ratio to SI570 register value
		RFREQ=0;					//higher 30 bit
		_RFREQ=0;					//lower 8 bit
		for (i=1;i<=34;i++)			//34 bit division RFREQ=vco/fcryst. Note that RFREQ only has 34 bits !=0
			{
			d=vco/fcryst_tot;
			if (i<=26) RFREQ=(RFREQ<<1)+d;
			else _RFREQ=(_RFREQ<<1)+d;
			vco=(vco-d*fcryst_tot)<<1;
			}
		for (i=5;i>=1;i--)
			{
			SI570_data[i]=rfreq.w[5-i];
			}
		SI570_data[1]=(SI570_data[1] & 0x3f)|(N<<6);
		SI570_data[0]=(N>>2) | (HS_DIV<<5);
		}							//division done
	return(valid);					//return 1 if valid register content found, else 0
}

void SI570_freezeNCO(void)  REWRITABLE;
void SI570_freezeNCO(void)
{
	I2CSendStart();					
	I2CSendByte(I2C_adr<<1);	    // send device address 
	I2CSendByte(137);			    // send Byte address 137
	I2CSendByte(0x10);	    		// send freeze cmd 0x10
	I2CSendStop();
}

void SI570_unfreezeNCO(void)  REWRITABLE;
void SI570_unfreezeNCO(void)
{
	I2CSendStart();					
	I2CSendByte(I2C_adr<<1);	    // send device address 
	I2CSendByte(137);			    // send Byte address 137
	I2CSendByte(0x00);	    		// send unfreeze cmd 0x00
	I2CSendStop();
}


void SI570_Load(void)  REWRITABLE;
void SI570_Load(void)
{
uchar i;
    SI570_freezeNCO();
	I2CSendStart();	
	I2CSendByte((I2C_adr<<1) & 0xFE);	// send device address 
	I2CSendByte(byte_adr);				// send Byte address 
	for (i=0;i<=5;i++)	I2CSendByte(SI570_data[i]);	 // send data 
	I2CSendStop();
	SI570_unfreezeNCO();
	I2CSendStart();	
	I2CSendByte((I2C_adr<<1) & 0xFE);	// send device address 
	I2CSendByte(135);					// send Byte address 
	I2CSendByte(0x40);					// send new frequency available 
	I2CSendStop();
}

void SI570_Load_NoFreeze(void)  REWRITABLE;
void SI570_Load_NoFreeze(void)
{
uchar i;
	I2CSendStart();	
	I2CSendByte((I2C_adr<<1) & 0xFE);	// send device address 
	I2CSendByte(byte_adr);				// send Byte address 
	for (i=0;i<=5;i++)	I2CSendByte(SI570_data[i]);	 // send data 
	I2CSendStop();
}

char SetFrequency(void) REWRITABLE;
char SetFrequency(void){
unsigned long fq;
unsigned long diff;
unsigned long scaled_fq;
unsigned long conv_fraction;
unsigned long acc;
// High level function to set the Si570 frequency, taking into account
// the VFO multiplier value, and sets the filter select lines appropriately 
fq = freq.qw;	// Remember frequency before applying scale factors

if (Si570_freq_offset != 0)
	freq.qw += Si570_freq_offset;	// Account for offset, if any
if (Si570_freq_mult > 1)
	freq.qw *= Si570_freq_mult;		// Account for VFO multiplier
if (Si570_freq_div > 1)
	freq.qw /= Si570_freq_div;		// Account for VFO divider

// SI570CalcRegs() wants freq.qw in MHz * 2^21, not in Hz. So we need
// to calculate (freq * 2^21) / 10^6 which is freq * 2.097152. To minimise run-time we
// first do a simple fixed-point multiplication by the fractional part, then add in freq*2.

scaled_fq = freq.qw;
for (conv_fraction = 417264663L, acc = 0L;
	 conv_fraction != 0;
	 conv_fraction <<= 1)
{
	if (conv_fraction & 0x80000000)
		acc += scaled_fq;
	scaled_fq >>= 1;
}
acc >>= 1;
freq.qw = (freq.qw << 1) + acc;		// freq.qw = freq.qw * 2.097152

// Calculate values to program into Si570
if (SI570CalcRegs() == 0)
	return -1;			// Give up if can't work out valid values

byte_adr = 7;  			// (Ignore supplied load address, we always load here!)

// Test if clickless tuning is possible - only works if frequency has changed by
// less than 3500ppm since last full reprogram *and* the dividers have not
// changed.

if (fq > Si570_centre)
	diff = fq - Si570_centre;		// Work out how much frequency has changed
else
	diff = Si570_centre - fq;

if (SI570_data[0] == Si570_centre_regs[0] &&
	(SI570_data[1] & 0xC0) == (Si570_centre_regs[1] & 0xC0) &&
	diff < 100000 && (diff * 300) < Si570_centre)
{
	// Small change
	SI570_Load_NoFreeze();
} else {
	// Large change.
	SI570_Load();
	Si570_centre = fq; // Remember freq. for last full Si570 reprogram
	Si570_centre_regs[0] = SI570_data[0]; // ...and corresponding divider settings
	Si570_centre_regs[1] = SI570_data[1];
}
return 1;
}

