// user interface to USB_SYNTH
// created Nov. 1st, 2008 by DG8SAQ
// Firmware version number: 4.0
//
//       M0PUB provided clickless tuning support and optimized bootloader
//       PE0FKO optimized I2C communication and streamlined parts of the code.
//#define DEBUG_LEDS		//switch debug LEDs support via I2C on/off

#define MAJ_VER 5
#define MIN_VER 0
// PB4, PB5 = custom lines
// PB3 = SCL
// PB1 = SDA
// PB0, PB2 = USB data lines



#define I2CPORT PORTB			// I2C is on Port B
#define I2C_DDR DDRB 
#define I2C_PIN PINB
#define BIT_SDA 1
#define BIT_SCL 3
#define SDA (1 << BIT_SDA)
#define SCL (1 << BIT_SCL)
#define USERP4 (1 << 4)
#define USERP5 (1 << 5)
#define RXTX USERP4
#define CWKEY (USERP5 | SDA)
#define PORTB_DDR 0			    //using PORTB, all port pins inputs
#define PORTB_INIT 0x30     	//pullups for USB pins PB4 and PB5

//EEPROM contents
// Define EEPROM locations for non-volatile data storage
#define F_CAL_VALUE 0x00	// AVR RC oscillator frequency calibration value
#define F_CRYST_STATUS 0x01 // Flag to indicate if Si570 reference frequency has been calibrated
#define F_CRYST 	0x02	// Si570 reference crystal frequency. Warning - 4 bytes long!
#define INIT_SI570	0x06	// I2C address of Si570 which enables store latest frequency and restore on init (default 0xFF=disabled)
#define INIT_FREQ 	0x07	// Frequency to restore on init (6 bytes)
#define I2C_ADR 	0x0D    // I2C address used in LCD mode
#define _A			0x80	// temp coefficent _a (4 bytes)
#define _B			0x84	// temp coefficent _b (4 bytes)
#define _C			0x88	// temp coefficent _c (4 bytes)
#define SAFETY_FLAG 0xFF    // not used in FSK firmware
#define EEPROM_BASE 0x100	//store frequency info in upper half of EEPROM
#define FREQ 0+EEPROM_BASE
#define OFS  4+EEPROM_BASE
#define MUL  8+EEPROM_BASE
#define DIV  9+EEPROM_BASE
#define DEFD 10+EEPROM_BASE

// Define range of allowed multipliers and dividers (and defaults) for the VFO frequency.
// The multiplier is typically used in systems which implicitly divide-down the VFO frequency
// again, such as some RF mixer architectures. The divider can be used in conjunction with the
// multiplier for systems that need a fractional convertion between specified frequency
// and actual VFO frequency.
#define MIN_FREQ_MULT 1
#define MAX_FREQ_MULT 99
#define DEF_FREQ_MULT 1

#define MIN_FREQ_DIV 1
#define MAX_FREQ_DIV 99
#define DEF_FREQ_DIV 1

typedef union 
{ 
   unsigned long qw; 
   unsigned char bytes[4]; 
} freq_t;

typedef union 
{ 
   signed long qw; 
   unsigned char bytes[4]; 
} ofs_t;

#include "I2CopenPE0FKO.h"			

#include "debug.h"
#include "setfreq.h"
#include "ini.h"

#include "SI570.h"			// Interface to SI570 device

extern char command;

#include "interface_fsk.h"

