/*---------------------------FunctionSetup(data)--------------------------------*/
/*	Called by USB interface code, via usbFunctionSetup() in main.c, when a      */
/*  control message has been received.                                          */
/*    Returns: 0xFF if 'user supplied function'- in our case FunctionWrite()    */
/*      - should be called.                                                     */
/*      Otherwise, number of bytes to return over USB link. Corresponding data  */
/*      is put into a buffer pointed to by usbMsgPtr                            */
/*                                                                              */
/*  (See MAIN.C and USBDRV.H for details)                                       */
/*  Note that most command processing is done here BUT commands which need      */
/*  access to the input data buffer need to be implemented in FunctionWrite     */
/*------------------------------------------------------------------------------*/

uchar FunctionSetup(uchar data[8]) REWRITABLE;
uchar FunctionSetup(uchar data[8])
{
    static uchar    replyBuf[6];
	uchar i;
	
    usbRequest_t *rq = (void *)data;
    usbMsgPtr = replyBuf;	
	if((rq->bRequest >= 0x30) &&
	  (rq->bRequest <=  0x38)){       		// use usbFunctionWrite to transfer data
		I2C_adr = data[2];	
		byte_adr= data[3];
		command=rq->bRequest;			
        return 0xff;
	}
	if(rq->bRequest == 0x3f){       		// SI570: read out frequency control registers
        I2CSendStart();						// read all registers in one block
        I2CSendByte(data[2]<<1);	    	// send device address from "value" lo
		I2CSendByte(7);			    		// send Byte address 7 of first register
		I2CSendStart();	
		I2CSendByte((data[2]<<1)|1);		// send device address from "value" lo for reading
		for (i=0;i<5;i++){
			replyBuf[i] = I2CReceiveByte();	// receive data byte
			}
		replyBuf[5] = I2CReceiveLastByte(); // receive without Acknowledge
        I2CSendStop(); 
 		for (i=0;i<6;i++){					// alternatively, read register after register
      	I2CSendStart();					
        I2CSendByte((data[2]<<1)&0xFE); 	// send device address from "value" low
		I2CSendByte(7+i);					// send Byte address from "value" high
		I2CSendStart();
		I2CSendByte((data[2]<<1)|1);		// send device address from "value" low, ready for read
        replyBuf[i] = I2CReceiveLastByte();	// receive data without Acknowledge and return
        I2CSendStop();
		}
        return 6;
	}
  	if(rq->bRequest == 0x41){       	// set/reset init freq status
        eeprom_write_byte((uint8_t *)INIT_SI570,data[2]); //status: low byte of value = I2C address
        return 0;
	}
	if (rq->bRequest == 0x90){			
		((int*)replyBuf)[0] = adc_lastval;// return ADC temperature data
		return 2;
		}
	if (rq->bRequest == 0x91){			
		((long*)replyBuf)[0] = _a;// return TCXO parameter _a
		return 4;
		}
	if (rq->bRequest == 0x92){			
		((long*)replyBuf)[0] = _b;// return TCXO parameter _b
		return 4;
		}
	if (rq->bRequest == 0x93){			
		((long*)replyBuf)[0] = _c;// return TCXO parameter _c
		return 4;
		}
	if (rq->bRequest == 0x94){
		_a=*(long *)&data[2];	// set TCXO parameter _a
		eeprom_write_byte((uint8_t *) _A, data[2]);
		eeprom_write_byte((uint8_t *) _A+1, data[3]);
		eeprom_write_byte((uint8_t *) _A+2, data[4]);
		eeprom_write_byte((uint8_t *) _A+3, data[5]);
		return 0;
		}
	if (rq->bRequest == 0x95){
		_b=*(long *)&data[2];	// set TCXO parameter _b
		eeprom_write_byte((uint8_t *) _B, data[2]);
		eeprom_write_byte((uint8_t *) _B+1, data[3]);
		eeprom_write_byte((uint8_t *) _B+2, data[4]);
		eeprom_write_byte((uint8_t *) _B+3, data[5]);
		return 0;
		}
	if (rq->bRequest == 0x96){
		_c=*(long *)&data[2];	// set TCXO parameter _c
		eeprom_write_byte((uint8_t *) _C, data[2]);
		eeprom_write_byte((uint8_t *) _C+1, data[3]);
		eeprom_write_byte((uint8_t *) _C+2, data[4]);
		eeprom_write_byte((uint8_t *) _C+3, data[5]);
		return 0;
		}
	if (rq->bRequest == 0x97){			
		((long*)replyBuf)[0] = fcryst;// return current fcryst
		return 4;
		}
	if (rq->bRequest == 0x98){
		UpdateFCryst();			// update crystal frequency from EEPROM and correct frequency
		if (SI570CalcRegs()>0) SI570_Load();
		return 0;
		}
	if(rq->bRequest == 0x99){ 
#ifdef DEBUG_LEDS        		// debug
		i2c_debug_outbyte(data[2]);	
#endif			
        return 0;
	}
	if (rq->bRequest == 0x9a){				// T-compensation on/off (0=on)
		block_TCXO=data[2];
		return 0;
		}
	if (rq->bRequest == 0x9b){				// T-compensation status (0=on)
		replyBuf[0] = block_TCXO;
		return 1;
		}
	replyBuf[0] = 0xff;						// return value 0xff => command not supported 
    return 1;
}


/*--------------------------FunctionWrite(data, len)----------------------------*/
/*	Called by USB interface code, via usbFunctionWrite() in main.c, when a      */
/*  packet has been received and FunctionSetup returns 0xFF                     */
/*    Returns: 0xFF if an error has occurred                                    */
/*             0 if more data is expected                                       */
/*             non-zero if all data received and processed                      */
/*                                                                              */
/*  (See MAIN.C and USBDRV.H for details)                                       */
/*------------------------------------------------------------------------------*/

void safefreq(void) REWRITABLE;
void safefreq(void)
{
uchar i;
if (eeprom_read_byte((uint8_t *)INIT_SI570)!=0xff) 	//  read init status for SI570
	{
	for (i=0;i<6;i++) eeprom_write_byte((uint8_t *)(i+INIT_FREQ),SI570_data[i]);
	}
}

uchar FunctionWrite(uchar *data, uchar len)  REWRITABLE;
uchar FunctionWrite(uchar *data, uchar len) 
{
uchar i;
	if (command==0x30)					// get register contents and load to SI570
		{		
		if (len<6) return 1;
		for (i=0;i<6;i++) SI570_data[i]=data[i];
		SI570_Load();
		safefreq();
		return 1;
		}

	if (len<4) return 1;

	if (command==0x32)					// get frequency and load to SI570
		{
		for (i=0;i<4;i++) freq.bytes[i]=data[i];
		byte_adr = 7;
		if (SI570CalcRegs()>0) SI570_Load();
		safefreq();
		return 1;
		}
	if (command==0x33)					// write new crystal frequency to EEPROM
		{	
		for (i=0;i<4;i++) eeprom_write_byte((uint8_t *)(F_CRYST+i), data[i]);
		eeprom_write_byte((uint8_t *)(F_CRYST_STATUS), 0);
		for (i=0;i<4;i++) fcr.bytes[i]=data[i];
		return 1;
		}
	if (command==0x34)					// get frequency and load to SI570
		{
		for (i=0;i<4;i++) freq.bytes[i]=data[i];
		SetFrequency();
		safefreq();
		return 1;
		} 
	return 1;
}
